﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using System.IO;
using System.Drawing.Imaging;

namespace howto_image_slices
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        // The result image.
        private Bitmap ResultImage = null;

        // Open a new file.
        private void mnuFileOpen_Click(object sender, EventArgs e)
        {
            ofdImage.Multiselect = true;
            if (ofdImage.ShowDialog() == DialogResult.OK)
            {
                foreach (string filename in ofdImage.FileNames)
                {
                    FileData file_data = new FileData(filename);
                    lstFiles.Items.Add(file_data);
                }

                ClearResult();
            }
        }

        // Save the result.
        private void mnuFileSave_Click(object sender, EventArgs e)
        {
            if (sfdImage.ShowDialog() == DialogResult.OK)
            {
                SaveImage(ResultImage, sfdImage.FileName);
            }
        }

        // Display the context menu.
        private void lstFiles_MouseDown(object sender, MouseEventArgs e)
        {
            // If this is not the right button, do nothing.
            if (e.Button != MouseButtons.Right) return;

            // Select the item under the mouse.
            lstFiles.SelectedIndex =
                lstFiles.IndexFromPoint(e.Location);

            // If no item is selected. do nothing.
            if (lstFiles.SelectedIndex == -1) return;

            // Display the context menu.
            ctxFile.Show(lstFiles, e.Location);
        }

        // Enable and disable the appropriate context menu items.
        private void ctxFile_Opening(object sender, CancelEventArgs e)
        {
            ctxMoveUp.Enabled =
                (lstFiles.SelectedIndex > 0);
            ctxMoveDown.Enabled =
                (lstFiles.SelectedIndex < lstFiles.Items.Count - 1);
        }

        // Move the selected file up in the list.
        private void ctxMoveUp_Click(object sender, EventArgs e)
        {
            // Get the selected index and its item.
            int index = lstFiles.SelectedIndex;
            if (index < 1) return;
            object item = lstFiles.Items[index];

            lstFiles.Items.RemoveAt(index);
            lstFiles.Items.Insert(index - 1, item);

            ClearResult();
        }

        // Move the selected file down in the list.
        private void ctxMoveDown_Click(object sender, EventArgs e)
        {
            // Get the selected index and its item.
            int index = lstFiles.SelectedIndex;
            if (index >= lstFiles.Items.Count) return;
            object item = lstFiles.Items[index];

            lstFiles.Items.RemoveAt(index);
            lstFiles.Items.Insert(index + 1, item);

            ClearResult();
        }

        // Remove this file from the list.
        private void ctxDelete_Click(object sender, EventArgs e)
        {
            lstFiles.Items.RemoveAt(lstFiles.SelectedIndex);
            ClearResult();
        }

        // Display the sliced image.
        private void ShowImage()
        {
            // Find the biggest image size.
            int width = 0;
            int height = 0;
            foreach (FileData file_data in lstFiles.Items)
            {
                if (width < file_data.Picture.Width)
                    width = file_data.Picture.Width;
                if (height < file_data.Picture.Height)
                    height = file_data.Picture.Height;
            }

            // Get the number of slices and the slice width.
            int num_slices = lstFiles.Items.Count;
            int slice_width = width / num_slices;

            // Make the result image.
            ResultImage = new Bitmap(width, height);
            using (Graphics gr = Graphics.FromImage(ResultImage))
            {
                gr.Clear(picResult.BackColor);

                // Draw the slices.
                for (int i = 0; i < num_slices; i++)
                {
                    int x = i * slice_width;
                    FileData file_data = (FileData)lstFiles.Items[i];
                    Rectangle rect = new Rectangle(x, 0, width, height);
                    gr.DrawImage(file_data.Picture,
                        rect, rect, GraphicsUnit.Pixel);
                }
            }

            // Display the result.
            picResult.Image = ResultImage;
            picResult.Visible = true;
            mnuFileSave.Enabled = true;
        }

        // Draw the image.
        private void btnDraw_Click(object sender, EventArgs e)
        {
            ShowImage();
        }

        // Save the file with the appropriate format.
        public void SaveImage(Image image, string filename)
        {
            string extension = Path.GetExtension(filename);
            switch (extension.ToLower())
            {
                case ".bmp":
                    image.Save(filename, ImageFormat.Bmp);
                    break;
                case ".exif":
                    image.Save(filename, ImageFormat.Exif);
                    break;
                case ".gif":
                    image.Save(filename, ImageFormat.Gif);
                    break;
                case ".jpg":
                case ".jpeg":
                    image.Save(filename, ImageFormat.Jpeg);
                    break;
                case ".png":
                    image.Save(filename, ImageFormat.Png);
                    break;
                case ".tif":
                case ".tiff":
                    image.Save(filename, ImageFormat.Tiff);
                    break;
                default:
                    throw new NotSupportedException(
                        "Unknown file extension " + extension);
            }
        }

        // Clear the result image and disable the Save menu item.
        private void ClearResult()
        {
            picResult.Image = null;
            picResult.Visible = false;
            mnuFileSave.Enabled = false;
        }
    }
}
